package utils;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

import STATICS.Helper;

public class UIGenerate extends JPanel implements MouseListener{

	private static final long serialVersionUID = 1L;
	
	private BufferedImage background=null;
	private BufferedImage foreground=null;
	private BufferedImage shadow=null;
	private BufferedImage cartSel=null;
	private BufferedImage cartShade=null;
	private BufferedImage cartStroke=null;
	private BufferedImage cart=null;
	
	private Vector<BufferedImage> labels=new Vector<BufferedImage>();
	private Vector<BufferedImage> facts=new Vector<BufferedImage>();
	
	private Vector<Rectangle>clickList=new Vector<Rectangle>();
	private Vector<Rectangle>clickLabelMap=new Vector<Rectangle>();
	
	
	private double[] scores={23,24,26,27};
//	private double[] scores={15,15,15,15};
	private int[] order={0,1,2,3};
	
	private int[] cartx={115,187,210,117};
	private int[] carty={134,54,295,249};
	private int[] anchx={123,201,216,203};
	private int[] anchy={185,106,242,179};
	
	private int selected=1;
	
	public UIGenerate(String titre) {
		
		
		generate(scores);
		this.addMouseListener(this);
		
		try {
			for(int i=1;i<=4;i++){
				labels.add(ImageIO.read(Helper.load("utils/images/c"+i+".png")));
				facts.add(ImageIO.read(Helper.load("utils/images/factc"+i+".png")));
			}
			foreground=ImageIO.read(Helper.load("utils/images/ui1.png"));
			background=ImageIO.read(Helper.load("utils/images/ui2.png"));
			shadow=ImageIO.read(Helper.load("utils/images/shadow.png"));
			cartShade=ImageIO.read(Helper.load("utils/images/cartouchedrop.png"));
			cart=ImageIO.read(Helper.load("utils/images/cartouche.png"));
			cartSel=ImageIO.read(Helper.load("utils/images/cartoucheselect.png"));
			cartStroke=ImageIO.read(Helper.load("utils/images/cartouchestroke.png"));
			this.setPreferredSize(new Dimension(foreground.getWidth(),foreground.getHeight()));
		} catch (IOException e){e.printStackTrace();}
	}
	
	
	public	void generate(double[] scores){
		for(int i=0;i<scores.length;i++){
			this.scores[i]=scores[i]*100;
		}
		findorder();
		//repaint();
	}	
	
	public void findorder() {
		double max=scores[0];
		for(int i=0;i<scores.length;i++){
			if(max<scores[i]){
				max=scores[i];
			}
		}
		int index=scores.length-1;
		for(int j=0;j<scores.length;j++){
			for(int i=0;i<scores.length;i++){
				if(max==scores[i]){
					order[i]=index;
					index--;
				}
			}
			double tmp=0;
			for(int i=0;i<scores.length;i++){
				if(scores[i]<max && scores[i]>tmp){
					tmp=scores[i];
				}
			}
			max=tmp;
		}

//		System.out.println();
//		for (double d : scores) {
//			System.out.print(Math.round(d)+"|");
//		}
	}
	
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2d=(Graphics2D)g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		
		//copie du masque d'arrière plan ..
		g2d.drawImage(background,0,0,this);
		
		
		g2d.setColor(Color.yellow);
		int dy=3;
		int dx=302;
		double minHeight=10;
		double bornesup=0;
		for (double score : scores){
//			if(score>maxscore){maxscore=score;}if(score<minscore){minscore=score;}
		}
		for (double score : scores){
			bornesup+=minHeight+score/2;
		}
		
		clickList.clear();
		
		for(int i=scores.length-1;i>=0;i--){
			for (int index=order.length-1; index>=0;index--) {
				if(order[index]==i){
					g2d.setColor(new Color(227,219,219));
					double height=minHeight+scores[index]/2;
					
					
//					double height=minHeight+(maxHeight-minHeight)*(itemsize-minscore)/(maxscore-minscore);
					double labelwidth=labels.get(index).getWidth()*(height/labels.get(index).getHeight());
					//label du plan
					BufferedImage buf=Helper.getFasterScaledInstance(cart, (int)labelwidth, (int)height, RenderingHints.VALUE_INTERPOLATION_BILINEAR, true);
					if(selected==i){
						g2d.setColor(Color.YELLOW);
						g2d.drawImage(facts.get(index),
								0,
								(int)(dy+height),
								this);
						g2d.drawImage(shadow,dx,(int)(dy+height-shadow.getHeight()/2), null);
						buf=Helper.getFasterScaledInstance(cartSel, (int)labelwidth, (int)height, RenderingHints.VALUE_INTERPOLATION_BILINEAR, true);
					}
					
					//dessins des labels dans le plan :
					
					BufferedImage shade=Helper.getFasterScaledInstance(
											cartShade,
											(int)(cartShade.getWidth()*(labelwidth/cart.getWidth())),
											(int)(cartShade.getHeight()*(labelwidth/cart.getWidth())),
											RenderingHints.VALUE_INTERPOLATION_BILINEAR,
											true);
					BufferedImage border=Helper.getFasterScaledInstance(
							cartStroke,
							(int)(buf.getWidth()+2),
							(int)(buf.getHeight()+2),
							RenderingHints.VALUE_INTERPOLATION_BILINEAR,
							true);
					g2d.drawImage(shade,(int)(cartx[index]-buf.getWidth()/2),(int)(carty[index]-buf.getHeight()/2),null);
					g2d.drawImage(border,(int)(cartx[index]-border.getWidth()/2),(int)(carty[index]-border.getHeight()/2),null);
//					triangle de pointage ...
					int[] xs = { (int)(cartx[index]-30*labelwidth/cart.getWidth()), (int)(cartx[index]+30*labelwidth/cart.getWidth()), anchx[index]};
					int[] ys = { carty[index], carty[index], anchy[index]};
					Polygon triangle = new Polygon(xs, ys, xs.length);
					
					g2d.fill(triangle);
					Color tmp=g2d.getColor();
					g2d.setColor(new Color(77, 77, 77));
					g2d.draw(triangle);
					g2d.setColor(tmp);
					
					
					g2d.drawImage(buf,(int)(cartx[index]-buf.getWidth()/2),(int)(carty[index]-buf.getHeight()/2),null);
					g2d.drawImage(
							Helper.getFasterScaledInstance(labels.get(index), (int)labelwidth, (int)height, RenderingHints.VALUE_INTERPOLATION_BILINEAR, true),
							(int)(cartx[index]-buf.getWidth()/2),
							(int)(carty[index]-buf.getHeight()/2),
							null
					);
//					
					clickList.add(0,new Rectangle(dx,dy,177,(int)height));
					clickLabelMap.add(0,new Rectangle((int)(cartx[index]-buf.getWidth()/2),(int)(carty[index]-buf.getHeight()/2),buf.getWidth(),buf.getHeight()));
					
					g2d.fill(clickList.firstElement());
					
					g2d.drawImage(
							Helper.getFasterScaledInstance(labels.get(index), (int)labelwidth, (int)height, RenderingHints.VALUE_INTERPOLATION_BILINEAR, true),
							(int)(dx+((clickList.firstElement().width-labelwidth)/2)),
							dy,
							null
					);
					
					g2d.setColor(new Color(77, 77, 77));
					g2d.draw(clickList.firstElement());
					dy+=(int)height;
					bornesup-=height;
					if(selected==i){
						dy=(int)(318-bornesup);
						g2d.drawImage(shadow,dx,(int)(dy-shadow.getHeight()/2), null);
					}
					break;
				}
			}
		}

		//copie du masque d'avant plan :
		
		g2d.drawImage(foreground,0,0,this);
		
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		for(int i=0;i<clickList.size();i++){
			if(clickList.get(i).contains(e.getPoint()) || clickLabelMap.get(i).contains(e.getPoint())){
				if(selected!=i){
					selected=i;repaint();
				}
			}
		}
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	@Override
	public void mousePressed(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
	}
}
