package controler;

import java.lang.reflect.Method;
import java.util.ArrayList;

import javax.swing.event.EventListenerList;

import STATICS.Helper;

import model.Model;
import model.ModelListener;

import view.View;
import view.ViewListener;

public abstract class Controler implements ViewListener,ModelListener,ControlerListener,ControlerInterface,Runnable{
	
	protected EventListenerList listeners=null;
	private boolean run=true;
	
	private ArrayList<View> registeredViews;
    private ArrayList<Model> registeredModels;
	
    public Controler(){
    	listeners=new EventListenerList();
		registeredViews=new ArrayList<View>();
		registeredModels=new ArrayList<Model>();
		Thread tmp=new Thread(this);
		tmp.setName("Controler");
		tmp.start();
    }
    
    public void dispose(){run=false;}
    
	@Override
	public void run() {
		while(run){
//			System.out.println("Controler alive ...");
			Helper.sleep(2000);
		}
	}
	
	public void addModel(Model model){
        registeredModels.add(model);
        for (View view: registeredViews) {
        	//autoinitView(view,model);
        	initView(view,model);
        }
        model.addListener(this);
    }

    public void removeModel(Model model){
        registeredModels.remove(model);
        model.removeListener(this);
    }

    public void addView(View view) {
        registeredViews.add(view);
        for (Model model: registeredModels) {
        	//autoinitView(view,model);
        	initView(view,model);
        }
        view.setVisible(true);
        view.addListener(this);
    }

    public void removeView(View view) {
        registeredViews.remove(view);
        view.removeListener(this);
        view.dispose();
        //s'il n'y a plus de vue, on etteind le système
        if(registeredViews.size()==0){
        	for (Model model : registeredModels) {model.dispose();}
        	this.dispose();
        }
    }
    
    public ArrayList<View> getViews(){return registeredViews;}
    public ArrayList<Model> getModels(){return registeredModels;}
    
	public void addListener(ControlerListener listener){listeners.add(ControlerListener.class,listener);}
	public void removeListener(ControlerListener listener){listeners.remove(ControlerListener.class,listener);}
	
	public void fireControlerEvent(ControlerEvent ev){
		for(ControlerListener listener : listeners.getListeners(ControlerListener.class)) {
			if(((ControlerListener)ev.getSource())!=ev)listener.update(ev);
		}
	}
	
	
	//little helpers .... 
	/**
	 * Sync the View to the model 
	 * if the model defines a method like "public ParamType getParam(){...}" 
	 * and a method "public void setParam(ParamType param){...}" exists in the view 
	 *  
	 * @param view The View to initiate
	 * @param model The Model to copy from
	 */
	protected void autoinitView(View view,Model model){
		for (Method modelMethod : model.getClass().getMethods()){
			if(modelMethod.getName().contains("get")){
				String propertyName=modelMethod.getName().substring(3);
				try {
					Object newValue=modelMethod.invoke(model);
					Method viewMethod = view.getClass().getMethod("set"+propertyName,new Class[] {newValue.getClass()});
					viewMethod.invoke(view,newValue);
				}catch(Exception ex){
				}
			}
		}
	}

	
	/*
	 * Convienence method that subclasses can call upon to fire off property changes
	 * back to the models and views. These method used reflection to inspect each of the model or view
	 * classes to determine if it is the owner of the property in question. If it
	 * isn't, a NoSuchMethodException is throws (which the method ignores).
	 */
	
	protected void setModelProperty(String propertyName) {
		setProperty(propertyName, registeredModels, null, null);	
	}
	protected void setModelProperty(String propertyName, Object newVal) {
		setProperty(propertyName, registeredModels, null, newVal);
	}
	protected void setModelProperty(String propertyName,Object oldVal,Object newVal){
		setProperty(propertyName, registeredModels, oldVal, newVal);
	}
	protected void setViewProperty(String propertyName) {
		setProperty(propertyName, registeredViews, null, null);	
	}
	protected void setViewProperty(String propertyName, Object newVal) {
		setProperty(propertyName, registeredViews, null, newVal);
	}
	protected void setViewProperty(String propertyName,Object oldVal,Object newVal){
		setProperty(propertyName, registeredViews, oldVal, newVal);
	}
	
	
	
	private void setProperty(String propertyName,ArrayList<?> invoke,Object oldVal,Object newVal){
		Class<?>[] param;
		if(newVal!=null && oldVal!=null){
			param=new Class<?>[]{oldVal.getClass(),newVal.getClass()};
		}else if(newVal==null && oldVal!=null){
			param=new Class<?>[]{oldVal.getClass()};
		}else if(newVal!=null && oldVal==null){
			param=new Class<?>[]{newVal.getClass()};
		}else{
			param=new Class<?>[]{};
		}
		
		for (Object obj: invoke) {
			Method method;
			try {
				method = obj.getClass().getMethod("set"+propertyName,param);
				try {
					if(newVal!=null && oldVal!=null){
						method.invoke(obj,oldVal,newVal);
					}else if(newVal==null && oldVal!=null){
						System.err.println("Raaaaah, il faut pas invoquer une mise a jour avec une oldVal !!");
					}else if(newVal!=null && oldVal==null){
						method.invoke(obj,newVal);
					}else{
						method.invoke(obj);
					}
				} catch (Exception ex) {
					ex.printStackTrace();
//					System.err.println("error "+ ex.getClass().getSimpleName());
//					System.err.println(method.toString()+" With " + newVal.toString());
				}
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			};
		}
	}
}