package model.user;

import java.awt.geom.Point2D;
import java.util.Vector;

import model.Model;
import model.user.exceptions.BreakOverlaps;
import model.user.exceptions.BreakPosNotFound;
import model.user.exceptions.NoBreakPosAtCurrentTime;
import model.user.exceptions.NoCurrentUserSetException;
import model.user.exceptions.NoSuchUserException;

public class UserModel extends Model{
	
	private String name;
	private String mapImage="";
	private double currentTime=0;
	private double updateInterval=0;
	private double range=0;
	private Vector<User> users=new Vector<User>();
	private User currentUser=null;
	private BreakPos currentBreak=null;
	
	public UserModel(String name){
		super();
		this.name=name;
		users.add(new User("Jim"));
		users.add(new User("Joe"));
		users.add(new User("Jack"));
		currentUser=users.lastElement();
	}
	
	//setters
	public void setCurrentUser(String newVal){
		for(User user : users) {
			if(user.getName().equals(newVal)){			
				currentUser=user;
				try {
					currentUser.setCurrentBreakPos(currentTime);
				} catch (NoBreakPosAtCurrentTime e) {
					System.err.println("no breakpos at current time for user "+ currentUser.getName());
				}
				break;
			}
		}
		fireModelEvent(new UserModelEvent(this,UserModelEventType.CurrentUser_set,currentUser));
	}
	public void setUserName(String newVal){setUserName(currentUser, newVal);}
	public void setUserName(User user, String newVal){
		if(user!=null && newVal.length()<=8 && users.contains(user)){
			for (User usr : users) {if(usr.getName().equals(newVal)){return;}}
			String oldName=user.getName();
			user.setName(newVal);
			fireModelEvent(new UserModelEvent(this,UserModelEventType.UserRename_set,oldName,user));
		}
	}
	public void setAddNewUser(String name){
		for(User user : users) {
			if(user.getName().equals(name)){setCurrentUser(currentUser.getName());return;}
		}
		users.add(new User(name));
		fireModelEvent(new UserModelEvent(this,UserModelEventType.AddNewUser_set,users.lastElement()));
	}
	
	public void setCurrentBreak(BreakPos pos){
		if(currentUser!=null && (currentBreak==null || !currentBreak.equals(pos))){
			for (User user : users) {
				if(user.getBreaks().contains(pos)){
					if(!currentUser.equals(user)){setCurrentUser(user.getName());}
					if(!pos.contain(currentTime)){setCurrentTime((pos.getStart()+pos.getEnd())/2);}
					currentBreak=pos;
					break;
				}
			}
		}
	}
	public void setMapImage(String newVal){
		mapImage=newVal;
		fireModelEvent(new UserModelEvent(this,UserModelEventType.MapImage_set,mapImage));
	}
	public void setCurrentTime(Double newVal) {
		double oldVal=currentTime;
		currentTime=Math.max(0, newVal);
		currentTime=Math.min(range, currentTime);
		setCurrentUserBreak(currentTime);
		fireModelEvent(new UserModelEvent(this,UserModelEventType.CurrentTime_set,(Double)oldVal,(Double)currentTime));
	}
	public void setRange(Double newValue) {
		this.range=newValue;
		fireModelEvent(new UserModelEvent(this,UserModelEventType.Range_set,(Double)range));
	}
	public void setUpdateInterval(Double newValue) {
		this.updateInterval=newValue;
		System.out.println("depart");
		fireModelEvent(new UserModelEvent(this,UserModelEventType.UpdateInterval_set,(Double)updateInterval));
	}
	public void setRemoveUser(String name){
		for(User user : users) {
			if(user.getName().equals(name)){
				users.remove(user);
				fireModelEvent(new UserModelEvent(this,UserModelEventType.RemoveUser_set,user));
				if(!users.isEmpty()){
					setCurrentUser(users.firstElement().getName());
				}else{
					currentUser=null;
					fireModelEvent(new UserModelEvent(this,UserModelEventType.NoMoreUsers_set));
				}
				break;
			}
		}
	}
	public void setCopyUser(String name){setCopyUser(currentUser, name);}
	public void setCopyUser(User user,String name){
		if(user!=null && !user.getName().equals(name) && users.contains(user)){
			users.add(new User(user,name));
			fireModelEvent(new UserModelEvent(this,UserModelEventType.AddNewUser_set,users.lastElement()));
		}
	}
	public void setAddBreakPosAtCurrentTime(Point2D.Double pos){
		if(currentUser!=null){
			try {
				currentUser.addBreakPos(pos.x, pos.y, currentTime, currentTime+10, "mon contexte");
				try {currentUser.setCurrentBreakPos(currentTime);
				} catch (NoBreakPosAtCurrentTime e) {e.printStackTrace();}
				fireModelEvent(new UserModelEvent(this,UserModelEventType.BreakPosAdded_set,currentUser.getCurrentBreak()));
			} catch (BreakOverlaps e) {System.err.println("pauses overlaps ...");}
		}
	}
	public void setRemoveBreakPos(BreakPos pos){
		if(currentUser!=null){
			try {
				currentUser.removeBreakPos(pos);
				fireModelEvent(new UserModelEvent(this,UserModelEventType.RemoveBreakPos_set,pos));
			} catch (BreakPosNotFound e) {
				System.err.println("hé, je ne pas trouvé la breakpos a virer !");
			}
		}
	}
	public void setRemoveCurrentBreakPos(){
		if(currentUser!=null){
			try {
				currentUser.removeCurrentBreakPos();
				fireModelEvent(new UserModelEvent(this,UserModelEventType.RemoveCurrentBreakPos_set));
			} catch (BreakPosNotFound e) {
				System.err.println("hé, je ne pas trouvé la breakpos a virer !");
			}
		}
	}

	public void setBreakPosLocation(Point2D.Double loc){
		if(currentUser!=null){
			setBreakPosLocation(currentUser.getCurrentBreak(),loc);
		}
	}
	
	public void setBreakPosDuration(Double t){
			if(currentUser!=null){
				setBreakPosDuration(currentUser.getCurrentBreak(),t);
			}
	}
	
	public void setBreakPosDuration(BreakPos pos, double t){
		if(pos!=null){
			//faire un certain nombre de vérification avant d'associer la nouvelle durée
			pos.setDuration(t);
			fireModelEvent(new UserModelEvent(this,UserModelEventType.BreakPosDuration_set,pos));
		}
	}
	
	public void setBreakPosDuration(double start,double end){
		if(currentUser!=null){
			setBreakPosDuration(currentUser.getCurrentBreak(),start,end);
		}
	}
	
	public void setBreakPosLocation(BreakPos pos, Point2D.Double loc){
		if(pos!=null){
			pos.setX(loc.getX());
			pos.setY(loc.getY());
			fireModelEvent(new UserModelEvent(this,UserModelEventType.BreakPosLocation_set,loc,pos));
		}
	}
	
	public void setBreakPosDuration(BreakPos pos, double start,double end){
		if(pos!=null){
			//faire un certain nombre de vérification avant d'associer les nouveaux temps..
			pos.setStart(start);
			pos.setEnd(end);
			fireModelEvent(new UserModelEvent(this,UserModelEventType.BreakPosDuration_set,pos));
		}
	}
	
	private void setCurrentUserBreak(Double time){
		if(currentUser!=null){
			BreakPos oldCurrent=currentUser.getCurrentBreak();
			try {
				currentUser.setCurrentBreakPos(time);
				//si les pauses sont différentes, on envoie un ordre de mise a jour..
				if(currentUser.getCurrentBreak()!=null && !currentUser.getCurrentBreak().equals(oldCurrent)){
					fireModelEvent(new UserModelEvent(this,UserModelEventType.CurrentBreakPos_set,currentUser.getCurrentBreak()));
				}
			} catch (NoBreakPosAtCurrentTime e) {
				if(oldCurrent!=null){
					fireModelEvent(new UserModelEvent(this,UserModelEventType.NoCurrentBreakPos_set));
				}
			}
		}
	}
	
	
	
	//getters
	public String getName(){return name;}
	public String getMapImage(){return mapImage;}
	public double getRange(){return range;}
	public double getUpdateInterval(){return updateInterval;}
	public double getCurrentTime(){return currentTime;}
	public Vector<User> getUsers(){return users;}
	public User getUserByName(String userName) throws NoSuchUserException{
		for (User user : users) {if(user.getName().equals(userName)){return user;}}
		throw new NoSuchUserException();
	}
	public User getCurrentUser() throws NoCurrentUserSetException{
		if(currentUser!=null){return currentUser;}
		throw new NoCurrentUserSetException();
	}
	
}
