package view.user;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.JPanel;

import model.user.BreakPos;
import STATICS.Helper;

class MapPanel extends JPanel implements MouseListener,MouseMotionListener{
	private static final long serialVersionUID = 1L;
	private UserView view;
	private BufferedImage background=null;
	
	private Hashtable<BreakPos, BreakDrawing> breakDrawings= new Hashtable<BreakPos, BreakDrawing>();
	private String currentUserName=null;
	private double currentTime=0;
	private BreakDrawing currentBDrawing=null;
	private Hashtable<String, Vector<BreakPos>>breaks=new Hashtable<String, Vector<BreakPos>>();
	
	public MapPanel(UserView view){
		super();
		this.view=view;
		this.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.RED),"Users Paths Display"));
		this.setPreferredSize(new Dimension(200,50));
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
	}
	
	public void setCurrentTime(double t){
		setCurrentBDrawing();
		currentTime=t;
		setCurrentBDrawing();
	}
	
	private void setCurrentBDrawing(){
		currentBDrawing=null;
		if(currentUserName!=null && breaks.get(currentUserName)!=null){
			for (BreakPos pos : breaks.get(currentUserName)) {
				if(pos.contain(currentTime)){
					currentBDrawing=breakDrawings.get(pos);
					repaint(breakDrawings.get(pos).getClip());
					break;
				}
			}
		}
	}
	
	public void setBreakPosLocation(BreakPos pos){
		setRemoveBreakPos(pos);
		setAddBreakPos(pos);
	}
	public void setBreakPosDuration(BreakPos pos){
		setRemoveBreakPos(pos);
		setAddBreakPos(pos);
	}
	public void setAddNewUser(String name){breaks.put(name,new Vector<BreakPos>());}
	public void setRemoveUser(String name){
		Vector<BreakPos>tmp=new Vector<BreakPos>();
		tmp.addAll(breaks.get(name));
		for (BreakPos pos : tmp) {setRemoveBreakPos(pos);}	
		breaks.remove(name);
	}
	public void setRemoveCurrentBreakPos(){
		for (BreakPos tmp : breaks.get(currentUserName)){
			if(tmp.contain(currentTime)){
				setRemoveBreakPos(tmp);
				break;
			}
		}
	}
	public void setRemoveBreakPos(BreakPos pos){
		BreakPos next=null,previous=null;
		for (BreakPos tmp : breaks.get(currentUserName) ) {
			if(tmp!=pos){
				if(tmp.getEnd()<pos.getStart()){
					if(previous!=null){if(previous.getEnd()<=tmp.getStart())previous=tmp;
					}else previous=tmp;
				}
				if(tmp.getStart()>=pos.getEnd()){
					if(next!=null){ if(next.getStart()>tmp.getEnd()) next=tmp;
					}else next=tmp;
				}
			}
		}
		Rectangle r=breakDrawings.get(pos).getClip();
		if(previous!=null)r=r.union(breakDrawings.get(previous).getClip());
		if(next!=null)r=r.union(breakDrawings.get(next).getClip());
		
		if(currentBDrawing!=null && currentBDrawing.equals(breakDrawings.get(pos))){currentBDrawing=null;}
		breakDrawings.remove(pos);
		(breaks.get(currentUserName)).remove(pos);
		repaint(r);
	}
	public void setUserName(String oldName, String newName){
		breaks.put(newName, breaks.get(oldName));
		breaks.remove(oldName);
	}
	public void setMapImage(String url){
		try {
			background=ImageIO.read(Helper.load(url));
			this.setPreferredSize(new Dimension(background.getWidth(),background.getHeight()));
			view.pack();
		} catch (IOException e){e.printStackTrace();}
	}
	public void setCurrentUser(String newVal) {
		Vector<BreakPos> tmp=null;
		if(currentUserName!=null && breaks.get(currentUserName)!=null){tmp=breaks.get(currentUserName);}
		currentUserName=newVal;
		//dessiner l'ancien en gris
		if(tmp!=null){
			for (BreakPos pos:tmp){
				int index= tmp.indexOf(pos);
				Rectangle clip=breakDrawings.get(pos).getClip();
				if(tmp.size()>1){
					if(!tmp.firstElement().equals(pos)){clip.add(breakDrawings.get(tmp.get(index-1)).getClip());}
					if(!tmp.lastElement().equals(pos)){clip.add(breakDrawings.get(tmp.get(index+1)).getClip());}
				}
				repaint(clip);
			}
		}
		//et le nouveau selectioné ..
		if(currentUserName!=null && breaks.get(currentUserName)!=null){
			tmp=breaks.get(currentUserName);
			for (BreakPos pos:breaks.get(currentUserName)) {
				int index= tmp.indexOf(pos);
				Rectangle clip=breakDrawings.get(pos).getClip();
				if(tmp.size()>1){
					if(!tmp.firstElement().equals(pos)){clip.add(breakDrawings.get(tmp.get(index-1)).getClip());}
					if(!tmp.lastElement().equals(pos)){clip.add(breakDrawings.get(tmp.get(index+1)).getClip());}
				}
				repaint(breakDrawings.get(pos).getClip());
			}
			setCurrentBDrawing();
		}
	}
	public void setAddBreakPos(BreakPos pos){
			if(currentUserName!=null){
				Vector<BreakPos> vec=breaks.get(currentUserName);
				boolean inserted=false;
				for(int i=0;i<vec.size();i++) {
					if(vec.get(i).getStart()>pos.getEnd()){
						vec.insertElementAt(pos, i);
						inserted=true;
						break;
					}
				}
				if(!inserted){vec.add(pos);}
				breakDrawings.put(pos,new BreakDrawing(pos));
				
				int index=vec.indexOf(pos);
				Rectangle clip=breakDrawings.get(pos).getClip();
				if(vec.size()>1){
					if(!vec.firstElement().equals(pos)){clip.add(breakDrawings.get(vec.get(index-1)).getClip());}
					if(!vec.lastElement().equals(pos)){clip.add(breakDrawings.get(vec.get(index+1)).getClip());}
				}
				repaint(clip);
				if(pos.contain(currentTime)){currentBDrawing=breakDrawings.get(pos);}
			}
	}
	
	//bon on passe au lourd, voici la methode paint ...
	protected void paintComponent(Graphics g){
		Graphics2D g2d = (Graphics2D)g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		Insets inset = this.getBorder().getBorderInsets(this);
		
		g2d.setColor(Color.WHITE);
//		g2d.setColor(Helper.randomCol());
		g2d.fillRect(0,0,this.getWidth(),this.getHeight());
		
		//dessin de l'image de fond
		g2d.drawImage(background,
				inset.left,inset.top,this.getWidth()-inset.right,this.getHeight()-inset.bottom,
				inset.left,inset.top,this.getWidth()-inset.right,this.getHeight()-inset.bottom,
				null);
		
		//dessin des pauses les utilisateurs non courant
		g2d.setColor(Color.GRAY);

		
		for (String name : breaks.keySet()){
			if(currentUserName!=null && !name.equals(currentUserName)){
				for (BreakPos pos : breaks.get(name)){
					if(!breaks.get(name).lastElement().equals(pos)){
						Point2D.Double c1,c2;
						c1=breakDrawings.get(pos).circle.center;
						c2=breakDrawings.get(breaks.get(name).elementAt(breaks.get(name).indexOf(pos)+1)).circle.center;
						g2d.drawLine((int)c1.x,(int) c1.y, (int)c2.x,(int) c2.y);
					}
				}
				for (BreakPos pos : breaks.get(name)){
					g2d.setColor(Helper.getAlphaColor(g2d.getColor(),120));
					g2d.fill(breakDrawings.get(pos).circle);
					g2d.setColor(Helper.getAlphaColor(g2d.getColor(),255));
					g2d.draw(breakDrawings.get(pos).circle);
				}
			}
		}
		
		//dessin des pauses pour l'utilisateur courant
		g2d.setColor(Color.YELLOW);
		if(currentUserName!=null && breaks.get(currentUserName)!=null){
			for (BreakPos pos : breaks.get(currentUserName)){
				if(!breaks.get(currentUserName).lastElement().equals(pos)){
					Point2D.Double c1,c2;
					c1=breakDrawings.get(pos).circle.center;
					c2=breakDrawings.get(breaks.get(currentUserName).elementAt(breaks.get(currentUserName).indexOf(pos)+1)).circle.center;
					g2d.drawLine((int)c1.x,(int) c1.y, (int)c2.x,(int) c2.y);
				}
			}
			
			for (BreakPos pos : breaks.get(currentUserName)){
				g2d.setColor(Helper.getAlphaColor(g2d.getColor(),120));
				
				g2d.fill(breakDrawings.get(pos).circle);
				g2d.setColor(Helper.getAlphaColor(g2d.getColor(),255));
				g2d.draw(breakDrawings.get(pos).circle);
				if(pos.contain(currentTime)){
					Color tmp=g2d.getColor();
					g2d.setColor(Color.RED);
					if(moverOver)g2d.setColor(Color.BLUE);
					g2d.fill(breakDrawings.get(pos).mover);
					g2d.setColor(Color.ORANGE);
					if(moverOver)g2d.setColor(Color.CYAN);
					g2d.draw(breakDrawings.get(pos).mover);
					g2d.setColor(Color.RED);
					if(growerOver)g2d.setColor(Color.BLUE);
					g2d.fill(breakDrawings.get(pos).grower);
					g2d.setColor(Color.ORANGE);
					if(growerOver)g2d.setColor(Color.CYAN);
					g2d.draw(breakDrawings.get(pos).grower);
					g2d.setColor(tmp);					
				}
			}
		}
	
	}

	//reactions aux evenemements sur le plan ...
	BreakPos posDragged=null;
	BreakPos posGrowed=null;
	
	@Override
	public void mouseClicked(MouseEvent e) {}
	@Override
	public void mouseEntered(MouseEvent e){}
	@Override
	public void mouseExited(MouseEvent e){}
	@Override
	public void mousePressed(MouseEvent e){
		BreakPos pos=null;
		for (BreakPos tmp : breakDrawings.keySet()) {
			if(breakDrawings.get(tmp).contains(e.getPoint())){
				pos=tmp;
				break;
			}
		}
		if(pos!=null){
			if(!breaks.get(currentUserName).contains(pos)){
				for (String name : breaks.keySet()){
					if(breaks.get(name).contains(pos)){
						view.updateChangeCurrentUser(name);
						view.updateCurrentBreak(pos);
					}
				}
			}else if(breakDrawings.get(pos)!=currentBDrawing){
				if(breaks.get(currentUserName).contains(pos)){
					view.updateCurrentBreak(pos);
				}
			}else if(breakDrawings.get(pos)==currentBDrawing){
				if(currentBDrawing.mover.contains(e.getPoint())){
					posDragged=pos;
				}else if(currentBDrawing.grower.contains(e.getPoint())){
					posGrowed=pos;
				}
			}
		}else{
			view.updateAddCurrentUserBreak(new Point2D.Double(e.getX(),e.getY()));
		}
	}
	@Override
	public void mouseReleased(MouseEvent e){
		if(posDragged!=null){posDragged=null;System.out.println("endMove");}
		if(posGrowed!=null){posGrowed=null;System.out.println("endGrow");}
	}
	@Override
	public void mouseDragged(MouseEvent e){
		if(currentBDrawing!=null){
			if (posDragged!=null){
				view.updateCurrentBreakLocation(new Point2D.Double(e.getPoint().getX(),e.getPoint().getY()));
			} else if (posGrowed!=null){
				//on passe le nouveaiu rayon correspondant à une durée de pose ..
				double time=(int)(Math.sqrt(Math.pow(e.getX()-posGrowed.getX(), 2)+Math.pow(e.getY()-posGrowed.getY(), 2)));
				System.out.println((int)time);
				view.updateCurrentBreakDuration((double)time);
			}
		}
	}
	
	boolean moverOver=false,growerOver=false;
	@Override
	public void mouseMoved(MouseEvent e){
		if(currentBDrawing!=null){
			if(currentBDrawing.mover.contains(e.getPoint()) && !moverOver){
				moverOver=true;
				repaint(currentBDrawing.mover.getBounds());
			}else if(!currentBDrawing.mover.contains(e.getPoint()) && moverOver){
				moverOver=false;
				repaint(currentBDrawing.mover.getBounds());
			}
			if(currentBDrawing.grower.contains(e.getPoint()) && !growerOver){
				growerOver=true;
				repaint(currentBDrawing.grower.getBounds());
			}else if(!currentBDrawing.grower.contains(e.getPoint()) && growerOver){
				growerOver=false;
				repaint(currentBDrawing.grower.getBounds());
			}
		}
//		if(currentBDrawing!=null){
//			if(currentBDrawing.mover.contains(e.getPoint()) && !moverOver){
//				moverOver=true;
//				repaint(breakDrawings.get(pos).mover.getBounds());
//		}
//		if(currentUserName!=null && breaks.get(currentUserName)!=null){
//			for (BreakPos pos : breaks.get(currentUserName)) {
//				if(breakDrawings.get(pos).mover.contains(e.getPoint()) && !moverOver){
//					moverOver=true;
//					repaint(breakDrawings.get(pos).mover.getBounds());
//				}else if(breakDrawings.get(pos).mover.contains(e.getPoint())){
//					
//				}
//				
//			}
//		}
	}
	
	class BreakDrawing{
		public Circle circle=new Circle();
		public Circle mover=new Circle();
		public Circle grower=new Circle();
		public final BreakPos pos;
		public final Color color=Helper.randomCol();
		
		public BreakDrawing(BreakPos pos) {
			this.pos=pos;
			circle.setCenter(pos.getPos());
			mover.setCenter(pos.getPos());
			circle.setRadius(pos.getEnd()-pos.getStart());
			mover.setRadius(4);
			grower.setRadius(4);
			grower.setCenter(new Point2D.Double((circle.center.x+circle.radius),circle.center.y));
		}
		public Rectangle getClip(){
			Rectangle rec=circle.getBounds();
			rec.add(grower.getBounds());
			rec.grow(2, 2);
			return rec;
		}
		public boolean contains(Point2D e){
			return circle.contains(e);
		}
	}

	class Circle extends Ellipse2D.Double{
		private static final long serialVersionUID = 1L;
		Point2D.Double center;
		double radius;
		
		public Circle(){
			center=new Point2D.Double(0,0);
			radius=0;
		}
		
		public Circle(Point2D.Double center,double radius){
			this();
			setRadius(radius);
			setCenter(center);
		}
		public void setCenter(Point2D.Double center){
			this.center=center;
			computeEllipse();
		}
		public void setRadius(double radius){
			this.radius=radius;
			computeEllipse();
		}
		
		private void computeEllipse(){setFrame(center.x-radius, center.y-radius, 2*radius, 2*radius);}
		
		@Override
		public boolean contains(Point2D p) {
			if(getBounds().contains(p)){return true;}
			return false;
		}
	}

}
