package view.user;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.event.WindowEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.plaf.basic.BasicComboPopup;
import javax.swing.plaf.basic.ComboPopup;
import javax.swing.plaf.metal.MetalComboBoxUI;
import model.user.BreakPos;



import STATICS.Helper;

import view.View;

public class UserView extends View{
	
	enum UserListActions{add,copy,rename,select;}
	
	private static final long serialVersionUID = 1L;
	
	private MapPanel mapPanel;
	private TimeLine timeLine;
	private UserSelect userSelect;
	private BreakSelect breakSelect;
	
	public UserView(){
		super();
		this.setLayout(new BorderLayout());
		userSelect=new UserSelect(this);
		breakSelect=new BreakSelect(this);
		JPanel tmp=new JPanel(new BorderLayout());
		tmp.add(userSelect,BorderLayout.WEST);
		tmp.add(breakSelect,BorderLayout.EAST);
		this.add(tmp,BorderLayout.NORTH);
		mapPanel=new MapPanel(this);
		this.add(mapPanel,BorderLayout.CENTER);
		timeLine=new TimeLine(this);
		this.add(timeLine,BorderLayout.SOUTH);
		
		this.pack();
		this.setResizable(false);
	}
	
	//interface publique pour le reglage de la vue (reactions aux evenements du controleur)
	public void setMapImage(String newVal){mapPanel.setMapImage(newVal);}
	public void setCurrentTime(Double newVal){
		timeLine.setCurrentTime(newVal);
		mapPanel.setCurrentTime(newVal);
		breakSelect.setCurrentTime(newVal);
	}
	public void setRange(Double newVal){timeLine.setRange(newVal.intValue());}
	public void setUpdateInterval(Double newVal){timeLine.setUpdateInterval(newVal.intValue());}
	public void setCurrentUser(String newVal){
		userSelect.setCurrentUser(newVal);
		breakSelect.setCurrentUser(newVal);
		mapPanel.setCurrentUser(newVal);
	}
	public void setUserName(String oldName, String newName){
		breakSelect.setUserName(oldName, newName);
		userSelect.setUserName(oldName, newName);
		mapPanel.setUserName(oldName, newName);
	}
	public void setAddBreakPos(BreakPos pos){
		mapPanel.setAddBreakPos(pos);
		breakSelect.setAddBreakPos(pos);
	}
	public void setBreakPosLocation(BreakPos pos){
		mapPanel.setBreakPosLocation(pos);
	}
	public void setBreakPosDuration(BreakPos pos){
		mapPanel.setBreakPosDuration(pos);
	}
	public void setAddNewUser(String name){
		userSelect.setAddNewUser(name);
		breakSelect.setAddNewUser(name);
		mapPanel.setAddNewUser(name);
	}
	public void setRemoveUser(String name){
		userSelect.setRemoveUser(name);
		breakSelect.setRemoveUser(name);
		mapPanel.setRemoveUser(name);
	}
	public void setRemoveCurrentBreakPos(){
		breakSelect.setRemoveCurrentBreakPos();
		mapPanel.setRemoveCurrentBreakPos();
	}
	public void setRemoveBreakPos(BreakPos pos){
		//Not sure wether this method is usefull or not ...
		System.out.println("effacer une position "+pos.toString());
	}
	
	
	//interface protected pour l'envoi de message depuis la vue vers le controleur
	protected void updateCurrentTime(double newVal){
		fireViewEvent(new UserViewEvent(this,UserViewEventType.CurrentTime_updated,newVal));
	}
	protected void updateCurrentUserName(String newVal){
		fireViewEvent(new UserViewEvent(this,UserViewEventType.UserName_updated,newVal));
	}
	protected void updateChangeCurrentUser(String newVal){
		fireViewEvent(new UserViewEvent(this,UserViewEventType.CurrentUser_updated,newVal));
	}
	protected void updateRemoveUser(String name){
		fireViewEvent(new UserViewEvent(this,UserViewEventType.RemoveUser_updated,name));
	}
	protected void updateAddNewUser(String name){
		fireViewEvent(new UserViewEvent(this,UserViewEventType.AddNewUser_updated,name));
	}
	protected void updateCopyCurrentUser(String copy){
		fireViewEvent(new UserViewEvent(this,UserViewEventType.CopyCurrentUser_updated,copy));
	}
	protected void updateCurrentBreak(BreakPos pos){
		fireViewEvent(new UserViewEvent(this,UserViewEventType.CurrentBreak_updated,pos));
	}
	protected void updateAddCurrentUserBreak(Point2D.Double pos){
		fireViewEvent(new UserViewEvent(this,UserViewEventType.AddCurrentUserBreak_updated,pos));
	}
	protected void updateCurrentBreakLocation(Point2D.Double pos){
		fireViewEvent(new UserViewEvent(this,UserViewEventType.CurrentUserBreakLocation_updated,pos));
	}
	protected void updateCurrentBreakDuration(Double t){
		fireViewEvent(new UserViewEvent(this,UserViewEventType.CurrentUserBreakDuration_updated,t));
	}
	protected void updateRemoveCurrentBreakPos(){
		fireViewEvent(new UserViewEvent(this,UserViewEventType.RemoveCurrentBreakPos_updated));
	}
	protected void updateRemoveBreakPos(BreakPos pos){
		fireViewEvent(new UserViewEvent(this,UserViewEventType.RemoveBreakPos_updated,pos));
	}
	
	//évènements sur la fenetre ....
	@Override
	public void windowClosing(WindowEvent e) {
		fireViewEvent(new UserViewEvent(this,UserViewEventType.View_closed,null,null));
	}
	
	
	class BreakSelect extends JPanel{
		private static final long serialVersionUID = 1L;
		private UserView view;
		private ToolButton erase,add;
		private double currentTime;
		private String currentUserName=null;
		private BreakPos currentBreakPos=null;
		
		private Hashtable<String, Vector<BreakPos>>breaks=new Hashtable<String, Vector<BreakPos>>();
		
		public BreakSelect(final UserView view) {
			super();
			this.view=view;
			erase=new ToolButton("view/user/images/erase.png"){
				private static final long serialVersionUID = 1L;
				@Override
				public void actionPerformed(ActionEvent e) {
					view.updateRemoveCurrentBreakPos();
				}
			};
			
			add=new ToolButton("view/user/images/add.png"){
				private static final long serialVersionUID = 1L;
				@Override
				public void actionPerformed(ActionEvent e){
					//BreakPos pos=new BreakPos(100, 100, currentTime, currentTime+10,"contexte");
					BreakPos next=null,previous=null;
					for (BreakPos tmp : breaks.get(currentUserName) ) {
						if(!tmp.contain(currentTime)){
							if(tmp.getEnd()<currentTime){
								if(previous!=null){if(previous.getEnd()<=tmp.getStart())previous=tmp;
								}else previous=tmp;
							}
							if(tmp.getStart()>currentTime+1){
								if(next!=null){ if(next.getStart()>tmp.getEnd()) next=tmp;
								}else next=tmp;
							}
						}
					}
					if(previous!=null && next!=null){
						double x=(previous.getX()+next.getX())/2;
						double y=(previous.getY()+next.getY())/2;
						System.out.println("repéré");
						view.updateAddCurrentUserBreak(new Point2D.Double(x,y));
					}else{
						if(previous!=null){next=previous;}
						if(next!=null){view.updateAddCurrentUserBreak(new Point2D.Double(next.getX(),next.getY()));}
						else{view.updateAddCurrentUserBreak(new Point2D.Double(0,0));}
					}
				}
			};
			
			this.setLayout(new FlowLayout(FlowLayout.LEFT,2,2));
			this.setBackground(Color.white);
			this.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.RED),"Breaks"));
			
			JPanel tmp=new JPanel();
			tmp.setBackground(Color.white);
			tmp.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
			
			tmp.add(erase);
			tmp.add(add);

			this.add(tmp);
			setVisibility();
		}
		
		public void setCurrentUser(String name){
			if(breaks.containsKey(name)) currentUserName=name;
			else currentUserName=null;
			setCurrentTime(currentTime);
		}
		public void setAddNewUser(String name){
			if(!breaks.containsKey(name))breaks.put(name,new Vector<BreakPos>());
			else setCurrentUser(name);
		}
		public void setRemoveUser(String name){
			Vector<BreakPos>tmp=new Vector<BreakPos>();
			tmp.addAll(breaks.get(name));
			for (BreakPos pos : tmp) {setRemoveBreakPos(pos);}
			breaks.remove(name);
			if(name.equals(currentUserName))setCurrentUser(name);
			setCurrentTime(currentTime);
		}
		public void setUserName(String oldName,String newName){
			if(breaks.containsKey(oldName)){
			 	breaks.put(newName, breaks.get(oldName));
			 	breaks.remove(oldName);
			}
		}
		
		public void setRemoveBreakPos(BreakPos pos){
			if(currentUserName!=null && breaks.get(currentUserName).contains(pos)){
				breaks.get(currentUserName).remove(pos);
				setCurrentTime(currentTime);
			}
		}
		public void setRemoveCurrentBreakPos(){
			if(currentBreakPos!=null){
				setRemoveBreakPos(currentBreakPos);
			}
		}
//		public void setCurrentBreakPos(BreakPos pos){
//			setVisibility();
//		}
		public void setAddBreakPos(BreakPos pos){
			if(currentUserName!=null && breaks.containsKey(currentUserName) && !breaks.get(currentUserName).contains(pos)){
				breaks.get(currentUserName).add(pos);
			}
			setCurrentTime(currentTime);
		}
		
		public void setCurrentTime(double currentTime){
			this.currentTime=currentTime;
			currentBreakPos=null;
			if(currentUserName!=null){
				for (BreakPos pos : breaks.get(currentUserName)){
					if(pos.contain(this.currentTime)){currentBreakPos=pos;break;}
				}
			}
			setVisibility();
		}
		
		private void setVisibility(){
			if(currentUserName!=null){
				if(currentBreakPos!=null){
					erase.setEnabled(true);
					add.setEnabled(false);
				}else{
					erase.setEnabled(false);
					add.setEnabled(true);
				}
			}else{
				add.setEnabled(false);
				erase.setEnabled(false);
			}
		}
		
	}
	
	class UserSelect extends JPanel{
		private static final long serialVersionUID = 1L;
		private UserView view;
		private UserList userList;
		private ToolButton erase,rename,copy,add;
		
		public UserSelect(final UserView view) {
			super();
			this.view=view;
			userList=new UserList();
			
			erase=new ToolButton("view/user/images/erase.png"){
				private static final long serialVersionUID = 1L;
				@Override
				public void actionPerformed(ActionEvent e) {
					view.updateRemoveUser((String)userList.getSelectedItem());
				}
			};
			copy=new ToolButton("view/user/images/copy.png"){
				private static final long serialVersionUID = 1L;
				@Override
				public void actionPerformed(ActionEvent e) {
					userList.setMode(UserListActions.copy);
					userList.setEditable(!userList.isEditable());
					userList.requestFocus();
				}
			};
			add=new ToolButton("view/user/images/add.png"){
				private static final long serialVersionUID = 1L;
				@Override
				public void actionPerformed(ActionEvent e) {
					userList.setMode(UserListActions.add);
					userList.setEditable(!userList.isEditable());
					userList.requestFocus();
				}
			};
			rename=new ToolButton("view/user/images/rename.png"){
				private static final long serialVersionUID = 1L;
				@Override
				public void actionPerformed(ActionEvent e) {
					userList.setMode(UserListActions.rename);
					userList.setEditable(!userList.isEditable());
					userList.requestFocus();
				}
			};

			
			this.setLayout(new FlowLayout(FlowLayout.LEFT,2,2));
			this.setBackground(Color.white);
			this.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.RED),"Users"));
			
			JPanel tmp=new JPanel();
			tmp.setBackground(Color.white);
			tmp.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
			
			tmp.add(userList);
			tmp.add(rename);
			tmp.add(erase);
			tmp.add(copy);
			tmp.add(add);
			noMoreUser(true);
			
			this.add(tmp);
			
		}
		
		
		public void setCurrentUser(String userName){userList.setCurrentUser(userName);}
		public void setUserName(String oldName,String newName){
			userList.setUserName(oldName, newName);
		}
		public void setAddNewUser(String name){userList.setAddNewUser(name);}
		public void setRemoveUser(String name){userList.setRemoveUser(name);}
		
		
		private void noMoreUser(boolean bool){rename.setEnabled(!bool);copy.setEnabled(!bool);erase.setEnabled(!bool);}
		
		
		
		class UserList extends JComboBox implements MouseWheelListener,ActionListener{

			
			private static final long serialVersionUID = 1L;
			private UserListUI userListUI=new UserListUI();
			private UserListActions mode=UserListActions.select;
			
			public UserList(){
				super();
				setUI(userListUI);
				addMouseWheelListener(this);
				addActionListener(this);
				setPreferredSize(new Dimension(100,27));
			}

			//setters de l'etat de la barre de selcetion
			public void setMode(UserListActions mode) {this.mode=mode;}
			public void setEditable(boolean bool){
				userListUI.setButtonVisible(!bool);
				super.setEditable(bool);
				switch (mode) {
				case add:getEditor().setItem("NAME"); getEditor().selectAll();break;
				case rename:getEditor().selectAll();break;
				case copy:getEditor().setItem("NEW_NAME");getEditor().selectAll();break;
				default:break;
				}
			}
			public void setCurrentUser(String userName){setSelectedItem(userName);}
			public void setUserName(String oldName,String newName){
				setRemoveUser(oldName);
				setAddNewUser(newName);
			}
			public void setAddNewUser(String name){
				addItem(name);
				//demande de mise a jour du modele en selectionnant
				//ce nouvel utilisateur comme utilisateur courant
				view.updateChangeCurrentUser(name);
				noMoreUser(false);
			}
			public void setRemoveUser(String name){
				removeItem(name);
				if(getItemCount()==0){noMoreUser(true);}
			}
			
			
			//reactions aux evenements
			@Override
			public void mouseWheelMoved(MouseWheelEvent e){
				if(getSelectedItem()!=null){
					int newIndex=getSelectedIndex();
					if(e.getWheelRotation()<0){if(getSelectedIndex()!=0){newIndex=getSelectedIndex()-1;}else{newIndex=getItemCount()-1;}
					}else{if(getSelectedIndex()!=getItemCount()-1){newIndex=getSelectedIndex()+1;}else{newIndex=0;}}
					view.updateChangeCurrentUser((String)getItemAt(newIndex));
				}
			}
			
			@Override
			public void actionPerformed(ActionEvent e){
				if(e.getActionCommand().equals("comboBoxChanged") && getSelectedItem()!=null){
					switch (mode) {
					case add:updateAddNewUser((String)getSelectedItem());view.updateChangeCurrentUser((String)getSelectedItem());break;
					case rename:view.updateCurrentUserName((String)getSelectedItem());break;
					case copy:view.updateCopyCurrentUser((String)getSelectedItem());;break;
					case select:view.updateChangeCurrentUser((String)getSelectedItem());break;
					default:break;
					}
					setEditable(false);
					mode=UserListActions.select;
				}
			}

			class UserListUI extends MetalComboBoxUI
			{
				BasicComboPopup popup;
				JButton arrow;

				public void setButtonVisible(boolean show){arrow.setEnabled(show);}
				public void setPopupVisible(boolean show){popup.setEnabled(show);}

				@Override
				protected ComboPopup createPopup(){
					popup = (BasicComboPopup)super.createPopup();
					return popup;
				}
				@Override
				protected JButton createArrowButton() {
					arrow=super.createArrowButton();
					return arrow;
				}
			}
		}
	}
	
	abstract class ToolButton extends JButton implements ActionListener{
		private static final long serialVersionUID = 1L;
		//private final View view;
		public ToolButton(String iconName){
			super();
			//this.view=view;
			setIcon(new ImageIcon(Helper.load(iconName)));
			setFocusPainted(false);
			setPreferredSize(new Dimension(27,27));
			addActionListener(this);
		}
	}
	
	class TimeLine extends JPanel implements MouseListener,MouseMotionListener{
		private static final long serialVersionUID = 1L;

		private UserView view;
		
		private final Color HOOVER_COLOR=Color.BLUE;
		private final int BAR_HEIGHT=15;
		private int range=15;
		private int updateInterval=4;
		
		private Cursor cur=new Cursor();
		private double cursorPos=5;
		private boolean hoover=false;
		private double ratio;
		private Rectangle in;
		
		public TimeLine(UserView view) {
			super();
			this.view=view;
			setPreferredSize(new Dimension(150,70));
			this.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.RED),"TimeLine"));
			view.pack();
			this.addMouseMotionListener(this);
			this.addMouseListener(this);
		}
		
		//setters pour les modifications venant du controleur
		public void setCurrentTime(double timeInstant){
			cursorPos=timeInstant;
			paintImmediately(cur.getClip());
			paintImmediately(cur.getClip());
		}
		public void setRange(int range){this.range=range;repaint();}
		public void setUpdateInterval(int value){this.updateInterval=value;repaint();}
		
		
		//reactions aux évènements souris...
		public void mouseClicked(MouseEvent arg0) {
			hoover=true;
			view.updateCurrentTime((arg0.getX()-in.getX())/ratio);
		}
		public void mouseEntered(MouseEvent arg0) {}
		public void mouseExited(MouseEvent arg0) {}
		public void mousePressed(MouseEvent arg0) {}
		public void mouseReleased(MouseEvent arg0) {}
		public void mouseDragged(MouseEvent arg0) {
			if(hoover){view.updateCurrentTime((arg0.getX()-in.getX())/ratio);}
		}
		public void mouseMoved(MouseEvent arg0){
			if(hoover && !cur.getClip().contains(arg0.getPoint())){
				hoover=false;
				repaint(cur.getClip());
			}
			if(!hoover && cur.getClip().contains(arg0.getPoint())){
				hoover=true;
				repaint(cur.getClip());
			}
		}
		
		protected void paintComponent(Graphics g){
			Graphics2D g2d = (Graphics2D)g;
			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
			Insets inset = this.getBorder().getBorderInsets(this);
			in=new Rectangle(inset.left+8,inset.top,this.getWidth()-inset.right-inset.left-16,this.getHeight()-inset.bottom-inset.top);
			
			g2d.setColor(Color.white);
			g2d.fillRect(0,0,this.getWidth(),this.getHeight());
			
			//essayer de faire un placement quel que soit le range 
			g2d.setColor(Color.DARK_GRAY);
			ratio=in.getWidth()/range;
			int h=5;
			for(int i=0;in.getWidth()>i*ratio;i=i+10){
				if(i%50==0){h=7;}else{h=5;}
				g2d.drawLine(
						(int)(i*ratio+in.getX()),
						(int)(in.getY()+BAR_HEIGHT-h),
						(int)(i*ratio+in.getX()),
						(int)(in.getY()+BAR_HEIGHT));
			}
			g2d.setColor(Color.GREEN);
			for(int i=0;in.getWidth()>i*ratio;i=i+updateInterval){
				g2d.drawLine(
						(int)(i*ratio+in.getX()),
						(int)(in.getY()+BAR_HEIGHT-10),
						(int)(i*ratio+in.getX()),
						(int)(in.getY()+BAR_HEIGHT));
				g2d.drawString("t"+((int)(i/updateInterval)), (int)(i*ratio+in.getX()-5), (int)in.getY()+h+BAR_HEIGHT-15);
				
			}
			g2d.setColor(Color.DARK_GRAY);
			g2d.drawLine(
					(int)(in.getX()-1+in.getWidth()),
					(int)(in.getY()+BAR_HEIGHT-7),
					(int)(in.getX()-1+in.getWidth()),
					(int)(in.getY()+BAR_HEIGHT));
			
			g2d.fill(new Rectangle2D.Double(in.getX(),in.getY()+BAR_HEIGHT,in.getWidth(),2));
			
			FontMetrics metrics = g2d.getFontMetrics();
			int adv = metrics.stringWidth(""+range);
			
			h=metrics.getHeight();
			g2d.drawString("0", (int)in.getX(), (int)in.getY()+h+BAR_HEIGHT-2);
			g2d.drawString(""+range, (int)(in.getX()+in.getWidth()-adv), (int)in.getY()+h+BAR_HEIGHT-2);

			
			
			if(hoover){g2d.setColor(HOOVER_COLOR);
			}else{g2d.setColor(Color.RED);}
			
			cur.setCursorPos(cursorPos);
			g2d.drawString(cur.getLabel(),cur.getLabelPos().x,cur.getLabelPos().y);
			g2d.fill(cur.getCursor());
			
		}
		
		//Classe interne pour le dessin d'un curseur + label de texte
		class Cursor {
			private static final long serialVersionUID = 1L;
			
			private Polygon cur=new Polygon();
			private Rectangle handle=new Rectangle();
			private Point textPos=new Point();
			private double cursorPos=0;
			private FontMetrics metrics;
			//normalement, il faudrait mesurer le texte
			private int textHeight=15,textWidth;
			
			public Cursor() {
				super();
				BufferedImage img=new BufferedImage(10,10,BufferedImage.TYPE_BYTE_GRAY);
				metrics=img.getGraphics().getFontMetrics();
			}
			
			public void setCursorPos(double x){
				this.cursorPos=x;
				cur.reset();
				cur.npoints=3;
				cur.xpoints=new int[]{(int)(ratio*x+in.getX()),(int)(ratio*x+in.getX())-6,(int)(ratio*x+in.getX())+6};
				cur.ypoints=new int[]{(int)in.getY()+BAR_HEIGHT,(int)in.getY()+BAR_HEIGHT-8,(int)in.getY()+BAR_HEIGHT-8};
				
				textWidth = metrics.stringWidth(getLabel());
				handle.setBounds((int)(ratio*x-textWidth/2+in.getX()), (int)in.getY()+BAR_HEIGHT-10-textHeight, textWidth, textHeight);
				textPos.x=(int)(ratio*x-textWidth/2+in.getX());
				textPos.y=(int)(in.getY()+BAR_HEIGHT-10);
			}
			
			public String getLabel(){return ""+(int)cursorPos;}
			public Polygon getCursor(){return cur;}
			public Point getLabelPos(){return textPos;}
			public Rectangle getHandle(){return handle;}
			
			public Rectangle getClip(){
				Rectangle clip=cur.getBounds();
				clip.add(handle);
				return clip;
			}
		}
	}	
}

